@@ -1,4 +1,4 @@
-/* $OpenBSD: ssl_methods.c,v 1.21 2020/12/01 07:46:02 tb Exp $ */
+/* $OpenBSD: ssl_methods.c,v 1.24 2021/03/31 16:59:32 tb Exp $ */
 /* Copyright (C) 1995-1998 Eric Young (eay@cryptsoft.com)
  * All rights reserved.
  *
@@ -59,12 +59,70 @@
 #include "ssl_locl.h"
 #include "tls13_internal.h"
 
+static const SSL_METHOD_INTERNAL DTLS_method_internal_data = {
+	.dtls = 1,
+	.server = 1,
+	.version = DTLS1_2_VERSION,
+	.min_tls_version = TLS1_1_VERSION,
+	.max_tls_version = TLS1_2_VERSION,
+	.ssl_new = dtls1_new,
+	.ssl_clear = dtls1_clear,
+	.ssl_free = dtls1_free,
+	.ssl_accept = ssl3_accept,
+	.ssl_connect = ssl3_connect,
+	.ssl_shutdown = ssl3_shutdown,
+	.ssl_renegotiate = ssl3_renegotiate,
+	.ssl_renegotiate_check = ssl3_renegotiate_check,
+	.ssl_pending = ssl3_pending,
+	.ssl_read_bytes = dtls1_read_bytes,
+	.ssl_write_bytes = dtls1_write_app_data_bytes,
+	.enc_flags = TLSV1_2_ENC_FLAGS,
+};
+
+static const SSL_METHOD DTLS_method_data = {
+	.ssl_dispatch_alert = dtls1_dispatch_alert,
+	.num_ciphers = ssl3_num_ciphers,
+	.get_cipher = dtls1_get_cipher,
+	.get_cipher_by_char = ssl3_get_cipher_by_char,
+	.put_cipher_by_char = ssl3_put_cipher_by_char,
+	.internal = &DTLS_method_internal_data,
+};
+
+static const SSL_METHOD_INTERNAL DTLS_client_method_internal_data = {
+	.dtls = 1,
+	.server = 0,
+	.version = DTLS1_2_VERSION,
+	.min_tls_version = TLS1_1_VERSION,
+	.max_tls_version = TLS1_2_VERSION,
+	.ssl_new = dtls1_new,
+	.ssl_clear = dtls1_clear,
+	.ssl_free = dtls1_free,
+	.ssl_accept = ssl_undefined_function,
+	.ssl_connect = ssl3_connect,
+	.ssl_shutdown = ssl3_shutdown,
+	.ssl_renegotiate = ssl3_renegotiate,
+	.ssl_renegotiate_check = ssl3_renegotiate_check,
+	.ssl_pending = ssl3_pending,
+	.ssl_read_bytes = dtls1_read_bytes,
+	.ssl_write_bytes = dtls1_write_app_data_bytes,
+	.enc_flags = TLSV1_2_ENC_FLAGS,
+};
+
+static const SSL_METHOD DTLS_client_method_data = {
+	.ssl_dispatch_alert = dtls1_dispatch_alert,
+	.num_ciphers = ssl3_num_ciphers,
+	.get_cipher = dtls1_get_cipher,
+	.get_cipher_by_char = ssl3_get_cipher_by_char,
+	.put_cipher_by_char = ssl3_put_cipher_by_char,
+	.internal = &DTLS_client_method_internal_data,
+};
+
 static const SSL_METHOD_INTERNAL DTLSv1_method_internal_data = {
 	.dtls = 1,
 	.server = 1,
 	.version = DTLS1_VERSION,
-	.min_version = DTLS1_VERSION,
-	.max_version = DTLS1_VERSION,
+	.min_tls_version = TLS1_1_VERSION,
+	.max_tls_version = TLS1_1_VERSION,
 	.ssl_new = dtls1_new,
 	.ssl_clear = dtls1_clear,
 	.ssl_free = dtls1_free,
@@ -92,8 +150,8 @@ static const SSL_METHOD_INTERNAL DTLSv1_client_method_internal_data = {
 	.dtls = 1,
 	.server = 0,
 	.version = DTLS1_VERSION,
-	.min_version = DTLS1_VERSION,
-	.max_version = DTLS1_VERSION,
+	.min_tls_version = TLS1_1_VERSION,
+	.max_tls_version = TLS1_1_VERSION,
 	.ssl_new = dtls1_new,
 	.ssl_clear = dtls1_clear,
 	.ssl_free = dtls1_free,
@@ -117,6 +175,64 @@ static const SSL_METHOD DTLSv1_client_method_data = {
 	.internal = &DTLSv1_client_method_internal_data,
 };
 
+static const SSL_METHOD_INTERNAL DTLSv1_2_method_internal_data = {
+	.dtls = 1,
+	.server = 1,
+	.version = DTLS1_2_VERSION,
+	.min_tls_version = TLS1_2_VERSION,
+	.max_tls_version = TLS1_2_VERSION,
+	.ssl_new = dtls1_new,
+	.ssl_clear = dtls1_clear,
+	.ssl_free = dtls1_free,
+	.ssl_accept = ssl3_accept,
+	.ssl_connect = ssl3_connect,
+	.ssl_shutdown = ssl3_shutdown,
+	.ssl_renegotiate = ssl3_renegotiate,
+	.ssl_renegotiate_check = ssl3_renegotiate_check,
+	.ssl_pending = ssl3_pending,
+	.ssl_read_bytes = dtls1_read_bytes,
+	.ssl_write_bytes = dtls1_write_app_data_bytes,
+	.enc_flags = TLSV1_2_ENC_FLAGS,
+};
+
+static const SSL_METHOD DTLSv1_2_method_data = {
+	.ssl_dispatch_alert = dtls1_dispatch_alert,
+	.num_ciphers = ssl3_num_ciphers,
+	.get_cipher = dtls1_get_cipher,
+	.get_cipher_by_char = ssl3_get_cipher_by_char,
+	.put_cipher_by_char = ssl3_put_cipher_by_char,
+	.internal = &DTLSv1_2_method_internal_data,
+};
+
+static const SSL_METHOD_INTERNAL DTLSv1_2_client_method_internal_data = {
+	.dtls = 1,
+	.server = 0,
+	.version = DTLS1_2_VERSION,
+	.min_tls_version = TLS1_2_VERSION,
+	.max_tls_version = TLS1_2_VERSION,
+	.ssl_new = dtls1_new,
+	.ssl_clear = dtls1_clear,
+	.ssl_free = dtls1_free,
+	.ssl_accept = ssl_undefined_function,
+	.ssl_connect = ssl3_connect,
+	.ssl_shutdown = ssl3_shutdown,
+	.ssl_renegotiate = ssl3_renegotiate,
+	.ssl_renegotiate_check = ssl3_renegotiate_check,
+	.ssl_pending = ssl3_pending,
+	.ssl_read_bytes = dtls1_read_bytes,
+	.ssl_write_bytes = dtls1_write_app_data_bytes,
+	.enc_flags = TLSV1_2_ENC_FLAGS,
+};
+
+static const SSL_METHOD DTLSv1_2_client_method_data = {
+	.ssl_dispatch_alert = dtls1_dispatch_alert,
+	.num_ciphers = ssl3_num_ciphers,
+	.get_cipher = dtls1_get_cipher,
+	.get_cipher_by_char = ssl3_get_cipher_by_char,
+	.put_cipher_by_char = ssl3_put_cipher_by_char,
+	.internal = &DTLSv1_2_client_method_internal_data,
+};
+
 const SSL_METHOD *
 DTLSv1_client_method(void)
 {
@@ -135,22 +251,40 @@ DTLSv1_server_method(void)
 	return &DTLSv1_method_data;
 }
 
+const SSL_METHOD *
+DTLSv1_2_client_method(void)
+{
+	return &DTLSv1_2_client_method_data;
+}
+
+const SSL_METHOD *
+DTLSv1_2_method(void)
+{
+	return &DTLSv1_2_method_data;
+}
+
+const SSL_METHOD *
+DTLSv1_2_server_method(void)
+{
+	return &DTLSv1_2_method_data;
+}
+
 const SSL_METHOD *
 DTLS_client_method(void)
 {
-	return DTLSv1_client_method();
+	return &DTLS_client_method_data;
 }
 
 const SSL_METHOD *
 DTLS_method(void)
 {
-	return DTLSv1_method();
+	return &DTLS_method_data;
 }
 
 const SSL_METHOD *
 DTLS_server_method(void)
 {
-	return DTLSv1_method();
+	return &DTLS_method_data;
 }
 
 #if defined(LIBRESSL_HAS_TLS1_3_CLIENT) && defined(LIBRESSL_HAS_TLS1_3_SERVER)
@@ -158,8 +292,8 @@ static const SSL_METHOD_INTERNAL TLS_method_internal_data = {
 	.dtls = 0,
 	.server = 1,
 	.version = TLS1_3_VERSION,
-	.min_version = TLS1_VERSION,
-	.max_version = TLS1_3_VERSION,
+	.min_tls_version = TLS1_VERSION,
+	.max_tls_version = TLS1_3_VERSION,
 	.ssl_new = tls1_new,
 	.ssl_clear = tls1_clear,
 	.ssl_free = tls1_free,
@@ -188,8 +322,8 @@ static const SSL_METHOD_INTERNAL TLS_legacy_method_internal_data = {
 	.dtls = 0,
 	.server = 1,
 	.version = TLS1_2_VERSION,
-	.min_version = TLS1_VERSION,
-	.max_version = TLS1_2_VERSION,
+	.min_tls_version = TLS1_VERSION,
+	.max_tls_version = TLS1_2_VERSION,
 	.ssl_new = tls1_new,
 	.ssl_clear = tls1_clear,
 	.ssl_free = tls1_free,
@@ -218,8 +352,8 @@ static const SSL_METHOD_INTERNAL TLS_client_method_internal_data = {
 	.dtls = 0,
 	.server = 0,
 	.version = TLS1_3_VERSION,
-	.min_version = TLS1_VERSION,
-	.max_version = TLS1_3_VERSION,
+	.min_tls_version = TLS1_VERSION,
+	.max_tls_version = TLS1_3_VERSION,
 	.ssl_new = tls1_new,
 	.ssl_clear = tls1_clear,
 	.ssl_free = tls1_free,
@@ -249,8 +383,8 @@ static const SSL_METHOD_INTERNAL TLS_legacy_client_method_internal_data = {
 	.dtls = 0,
 	.server = 0,
 	.version = TLS1_2_VERSION,
-	.min_version = TLS1_VERSION,
-	.max_version = TLS1_2_VERSION,
+	.min_tls_version = TLS1_VERSION,
+	.max_tls_version = TLS1_2_VERSION,
 	.ssl_new = tls1_new,
 	.ssl_clear = tls1_clear,
 	.ssl_free = tls1_free,
@@ -279,8 +413,8 @@ static const SSL_METHOD_INTERNAL TLSv1_method_internal_data = {
 	.dtls = 0,
 	.server = 1,
 	.version = TLS1_VERSION,
-	.min_version = TLS1_VERSION,
-	.max_version = TLS1_VERSION,
+	.min_tls_version = TLS1_VERSION,
+	.max_tls_version = TLS1_VERSION,
 	.ssl_new = tls1_new,
 	.ssl_clear = tls1_clear,
 	.ssl_free = tls1_free,
@@ -308,8 +442,8 @@ static const SSL_METHOD_INTERNAL TLSv1_client_method_internal_data = {
 	.dtls = 0,
 	.server = 0,
 	.version = TLS1_VERSION,
-	.min_version = TLS1_VERSION,
-	.max_version = TLS1_VERSION,
+	.min_tls_version = TLS1_VERSION,
+	.max_tls_version = TLS1_VERSION,
 	.ssl_new = tls1_new,
 	.ssl_clear = tls1_clear,
 	.ssl_free = tls1_free,
@@ -337,8 +471,8 @@ static const SSL_METHOD_INTERNAL TLSv1_1_method_internal_data = {
 	.dtls = 0,
 	.server = 1,
 	.version = TLS1_1_VERSION,
-	.min_version = TLS1_1_VERSION,
-	.max_version = TLS1_1_VERSION,
+	.min_tls_version = TLS1_1_VERSION,
+	.max_tls_version = TLS1_1_VERSION,
 	.ssl_new = tls1_new,
 	.ssl_clear = tls1_clear,
 	.ssl_free = tls1_free,
@@ -366,8 +500,8 @@ static const SSL_METHOD_INTERNAL TLSv1_1_client_method_internal_data = {
 	.dtls = 0,
 	.server = 0,
 	.version = TLS1_1_VERSION,
-	.min_version = TLS1_1_VERSION,
-	.max_version = TLS1_1_VERSION,
+	.min_tls_version = TLS1_1_VERSION,
+	.max_tls_version = TLS1_1_VERSION,
 	.ssl_new = tls1_new,
 	.ssl_clear = tls1_clear,
 	.ssl_free = tls1_free,
@@ -395,8 +529,8 @@ static const SSL_METHOD_INTERNAL TLSv1_2_method_internal_data = {
 	.dtls = 0,
 	.server = 1,
 	.version = TLS1_2_VERSION,
-	.min_version = TLS1_2_VERSION,
-	.max_version = TLS1_2_VERSION,
+	.min_tls_version = TLS1_2_VERSION,
+	.max_tls_version = TLS1_2_VERSION,
 	.ssl_new = tls1_new,
 	.ssl_clear = tls1_clear,
 	.ssl_free = tls1_free,
@@ -424,8 +558,8 @@ static const SSL_METHOD_INTERNAL TLSv1_2_client_method_internal_data = {
 	.dtls = 0,
 	.server = 0,
 	.version = TLS1_2_VERSION,
-	.min_version = TLS1_2_VERSION,
-	.max_version = TLS1_2_VERSION,
+	.min_tls_version = TLS1_2_VERSION,
+	.max_tls_version = TLS1_2_VERSION,
 	.ssl_new = tls1_new,
 	.ssl_clear = tls1_clear,
 	.ssl_free = tls1_free,
@@ -566,6 +700,8 @@ ssl_get_method(uint16_t version)
 		return (TLSv1_method());
 	if (version == DTLS1_VERSION)
 		return (DTLSv1_method());
+	if (version == DTLS1_2_VERSION)
+		return (DTLSv1_2_method());
 
 	return (NULL);
 }
