@@ -1,4 +1,4 @@
-/* $OpenBSD: ssl_versions.c,v 1.9 2020/08/09 16:30:29 jsing Exp $ */
+/* $OpenBSD: ssl_versions.c,v 1.14 2021/03/17 17:43:31 jsing Exp $ */
 /*
  * Copyright (c) 2016, 2017 Joel Sing <jsing@openbsd.org>
  *
@@ -193,7 +193,9 @@ test_ssl_enabled_version_range(void)
 	int failed = 1;
 	size_t i;
 
-	if ((ssl_ctx = SSL_CTX_new(TLS_method())) == NULL) { 
+	fprintf(stderr, "INFO: starting enabled version range tests...\n");
+
+	if ((ssl_ctx = SSL_CTX_new(TLS_method())) == NULL) {
 		fprintf(stderr, "SSL_CTX_new() returned NULL\n");
 		goto failure;
 	}
@@ -212,10 +214,10 @@ test_ssl_enabled_version_range(void)
 		SSL_set_options(ssl, vrt->options);
 
 		minver = maxver = 0xffff;
-		ssl->internal->min_version = vrt->minver;
-		ssl->internal->max_version = vrt->maxver;
+		ssl->internal->min_tls_version = vrt->minver;
+		ssl->internal->max_tls_version = vrt->maxver;
 
-		if (ssl_enabled_version_range(ssl, &minver, &maxver) != 1) {
+		if (ssl_enabled_tls_version_range(ssl, &minver, &maxver) != 1) {
 			if (vrt->want_minver != 0 || vrt->want_maxver != 0) {
 				fprintf(stderr, "FAIL: test %zu - failed but "
 				    "wanted non-zero versions\n", i);
@@ -405,21 +407,69 @@ static struct shared_version_test shared_version_tests[] = {
 		.want_maxver = TLS1_1_VERSION,
 	},
 	{
-		.ssl_method = DTLSv1_method,
+		.ssl_method = DTLS_method,
 		.options = 0,
-		.minver = TLS1_VERSION,
+		.minver = TLS1_1_VERSION,
 		.maxver = TLS1_2_VERSION,
 		.peerver = DTLS1_VERSION,
 		.want_maxver = DTLS1_VERSION,
 	},
+	{
+		.ssl_method = DTLS_method,
+		.options = 0,
+		.minver = TLS1_1_VERSION,
+		.maxver = TLS1_2_VERSION,
+		.peerver = DTLS1_2_VERSION,
+		.want_maxver = DTLS1_2_VERSION,
+	},
+	{
+		.ssl_method = DTLS_method,
+		.options = 0,
+		.minver = TLS1_1_VERSION,
+		.maxver = TLS1_2_VERSION,
+		.peerver = 0xfefc,	/* DTLSv1.3, probably. */
+		.want_maxver = DTLS1_2_VERSION,
+	},
 	{
 		.ssl_method = DTLSv1_method,
 		.options = 0,
-		.minver = TLS1_VERSION,
+		.minver = TLS1_1_VERSION,
+		.maxver = TLS1_1_VERSION,
+		.peerver = DTLS1_2_VERSION,
+		.want_maxver = DTLS1_VERSION,
+	},
+	{
+		.ssl_method = DTLSv1_2_method,
+		.options = 0,
+		.minver = TLS1_2_VERSION,
 		.maxver = TLS1_2_VERSION,
+		.peerver = DTLS1_2_VERSION,
+		.want_maxver = DTLS1_2_VERSION,
+	},
+	{
+		.ssl_method = DTLSv1_method,
+		.options = 0,
+		.minver = TLS1_1_VERSION,
+		.maxver = TLS1_1_VERSION,
 		.peerver = TLS1_2_VERSION,
 		.want_maxver = 0,
 	},
+	{
+		.ssl_method = DTLS_method,
+		.options = SSL_OP_NO_DTLSv1,
+		.minver = TLS1_1_VERSION,
+		.maxver = TLS1_2_VERSION,
+		.peerver = DTLS1_VERSION,
+		.want_maxver = 0,
+	},
+	{
+		.ssl_method = DTLS_method,
+		.options = SSL_OP_NO_DTLSv1_2,
+		.minver = TLS1_1_VERSION,
+		.maxver = TLS1_2_VERSION,
+		.peerver = DTLS1_2_VERSION,
+		.want_maxver = DTLS1_VERSION,
+	},
 };
 
 #define N_SHARED_VERSION_TESTS \
@@ -437,10 +487,12 @@ test_ssl_max_shared_version(void)
 
 	failed = 0;
 
+	fprintf(stderr, "INFO: starting max shared version tests...\n");
+
 	for (i = 0; i < N_SHARED_VERSION_TESTS; i++) {
 		svt = &shared_version_tests[i];
 
-		if ((ssl_ctx = SSL_CTX_new(svt->ssl_method())) == NULL) { 
+		if ((ssl_ctx = SSL_CTX_new(svt->ssl_method())) == NULL) {
 			fprintf(stderr, "SSL_CTX_new() returned NULL\n");
 			return 1;
 		}
@@ -454,13 +506,14 @@ test_ssl_max_shared_version(void)
 		SSL_set_options(ssl, svt->options);
 
 		maxver = 0;
-		ssl->internal->min_version = svt->minver;
-		ssl->internal->max_version = svt->maxver;
+		ssl->internal->min_tls_version = svt->minver;
+		ssl->internal->max_tls_version = svt->maxver;
 
-		if (ssl_max_shared_version(ssl, svt->peerver, &maxver) != 1) {
+		if (!ssl_max_shared_version(ssl, svt->peerver, &maxver)) {
 			if (svt->want_maxver != 0) {
 				fprintf(stderr, "FAIL: test %zu - failed but "
-				    "wanted non-zero shared version\n", i);
+				    "wanted non-zero shared version (peer %x)\n",
+				    i, svt->peerver);
 				failed++;
 			}
 			continue;
@@ -485,6 +538,8 @@ struct min_max_version_test {
 	const uint16_t maxver;
 	const uint16_t want_minver;
 	const uint16_t want_maxver;
+	const int want_min_fail;
+	const int want_max_fail;
 };
 
 static struct min_max_version_test min_max_version_tests[] = {
@@ -492,28 +547,28 @@ static struct min_max_version_test min_max_version_tests[] = {
 		.ssl_method = TLS_method,
 		.minver = 0,
 		.maxver = 0,
-		.want_minver = TLS1_VERSION,
-		.want_maxver = TLS1_3_VERSION,
+		.want_minver = 0,
+		.want_maxver = 0,
 	},
 	{
 		.ssl_method = TLS_method,
 		.minver = TLS1_VERSION,
 		.maxver = 0,
 		.want_minver = TLS1_VERSION,
-		.want_maxver = TLS1_3_VERSION,
+		.want_maxver = 0,
 	},
 	{
 		.ssl_method = TLS_method,
 		.minver = 0,
 		.maxver = TLS1_2_VERSION,
-		.want_minver = TLS1_VERSION,
+		.want_minver = 0,
 		.want_maxver = TLS1_2_VERSION,
 	},
 	{
 		.ssl_method = TLS_method,
 		.minver = 0,
 		.maxver = TLS1_3_VERSION,
-		.want_minver = TLS1_VERSION,
+		.want_minver = 0,
 		.want_maxver = TLS1_3_VERSION,
 	},
 	{
@@ -528,56 +583,54 @@ static struct min_max_version_test min_max_version_tests[] = {
 		.minver = TLS1_1_VERSION,
 		.maxver = 0,
 		.want_minver = TLS1_1_VERSION,
-		.want_maxver = TLS1_3_VERSION,
+		.want_maxver = 0,
 	},
 	{
 		.ssl_method = TLS_method,
 		.minver = TLS1_2_VERSION,
 		.maxver = 0,
 		.want_minver = TLS1_2_VERSION,
-		.want_maxver = TLS1_3_VERSION,
+		.want_maxver = 0,
 	},
 	{
 		.ssl_method = TLS_method,
 		.minver = 0x0300,
 		.maxver = 0,
 		.want_minver = TLS1_VERSION,
-		.want_maxver = TLS1_3_VERSION,
+		.want_maxver = 0,
 	},
 	{
 		.ssl_method = TLS_method,
 		.minver = 0x0305,
 		.maxver = 0,
-		.want_minver = 0,
-		.want_maxver = 0,
+		.want_min_fail = 1,
 	},
 	{
 		.ssl_method = TLS_method,
 		.minver = 0,
 		.maxver = 0x0305,
-		.want_minver = TLS1_VERSION,
+		.want_minver = 0,
 		.want_maxver = TLS1_3_VERSION,
 	},
 	{
 		.ssl_method = TLS_method,
 		.minver = 0,
 		.maxver = TLS1_1_VERSION,
-		.want_minver = TLS1_VERSION,
+		.want_minver = 0,
 		.want_maxver = TLS1_1_VERSION,
 	},
 	{
 		.ssl_method = TLS_method,
 		.minver = 0,
 		.maxver = TLS1_VERSION,
-		.want_minver = TLS1_VERSION,
+		.want_minver = 0,
 		.want_maxver = TLS1_VERSION,
 	},
 	{
 		.ssl_method = TLS_method,
 		.minver = 0,
 		.maxver = 0x0300,
-		.want_minver = 0,
-		.want_maxver = 0,
+		.want_max_fail = 1,
 	},
 	{
 		.ssl_method = TLS_method,
@@ -585,13 +638,14 @@ static struct min_max_version_test min_max_version_tests[] = {
 		.maxver = TLS1_1_VERSION,
 		.want_minver = TLS1_2_VERSION,
 		.want_maxver = 0,
+		.want_max_fail = 1,
 	},
 	{
 		.ssl_method = TLSv1_1_method,
 		.minver = 0,
 		.maxver = 0,
-		.want_minver = TLS1_1_VERSION,
-		.want_maxver = TLS1_1_VERSION,
+		.want_minver = 0,
+		.want_maxver = 0,
 	},
 	{
 		.ssl_method = TLSv1_1_method,
@@ -606,6 +660,7 @@ static struct min_max_version_test min_max_version_tests[] = {
 		.maxver = 0,
 		.want_minver = 0,
 		.want_maxver = 0,
+		.want_min_fail = 1,
 	},
 	{
 		.ssl_method = TLSv1_1_method,
@@ -613,26 +668,62 @@ static struct min_max_version_test min_max_version_tests[] = {
 		.maxver = TLS1_VERSION,
 		.want_minver = 0,
 		.want_maxver = 0,
+		.want_max_fail = 1,
+	},
+	{
+		.ssl_method = DTLS_method,
+		.minver = 0,
+		.maxver = 0,
+		.want_minver = 0,
+		.want_maxver = 0,
+	},
+	{
+		.ssl_method = DTLS_method,
+		.minver = 0,
+		.maxver = DTLS1_VERSION,
+		.want_minver = 0,
+		.want_maxver = DTLS1_VERSION,
+	},
+	{
+		.ssl_method = DTLS_method,
+		.minver = DTLS1_VERSION,
+		.maxver = 0,
+		.want_minver = DTLS1_VERSION,
+		.want_maxver = 0,
+	},
+	{
+		.ssl_method = DTLS_method,
+		.minver = DTLS1_VERSION,
+		.maxver = DTLS1_2_VERSION,
+		.want_minver = DTLS1_VERSION,
+		.want_maxver = DTLS1_2_VERSION,
 	},
 	{
 		.ssl_method = DTLSv1_method,
 		.minver = 0,
 		.maxver = 0,
-		.want_minver = DTLS1_VERSION,
-		.want_maxver = DTLS1_VERSION,
+		.want_minver = 0,
+		.want_maxver = 0,
 	},
 	{
 		.ssl_method = DTLSv1_method,
 		.minver = DTLS1_VERSION,
 		.maxver = 0,
 		.want_minver = DTLS1_VERSION,
-		.want_maxver = DTLS1_VERSION,
+		.want_maxver = 0,
 	},
 	{
 		.ssl_method = DTLSv1_method,
 		.minver = 0,
 		.maxver = DTLS1_VERSION,
-		.want_minver = DTLS1_VERSION,
+		.want_minver = 0,
+		.want_maxver = DTLS1_VERSION,
+	},
+	{
+		.ssl_method = DTLSv1_method,
+		.minver = 0,
+		.maxver = DTLS1_2_VERSION,
+		.want_minver = 0,
 		.want_maxver = DTLS1_VERSION,
 	},
 	{
@@ -641,6 +732,8 @@ static struct min_max_version_test min_max_version_tests[] = {
 		.maxver = TLS1_2_VERSION,
 		.want_minver = 0,
 		.want_maxver = 0,
+		.want_min_fail = 1,
+		.want_max_fail = 1,
 	},
 };
 
@@ -658,24 +751,26 @@ test_ssl_min_max_version(void)
 
 	failed = 0;
 
+	fprintf(stderr, "INFO: starting min max version tests...\n");
+
 	for (i = 0; i < N_MIN_MAX_VERSION_TESTS; i++) {
 		mmvt = &min_max_version_tests[i];
 
-		if ((ssl_ctx = SSL_CTX_new(mmvt->ssl_method())) == NULL) { 
+		if ((ssl_ctx = SSL_CTX_new(mmvt->ssl_method())) == NULL) {
 			fprintf(stderr, "SSL_CTX_new() returned NULL\n");
 			return 1;
 		}
 
-		if (SSL_CTX_set_min_proto_version(ssl_ctx, mmvt->minver) != 1) {
-			if (mmvt->want_minver != 0) {
+		if (!SSL_CTX_set_min_proto_version(ssl_ctx, mmvt->minver)) {
+			if (!mmvt->want_min_fail) {
 				fprintf(stderr, "FAIL: test %zu - failed to set "
 				    "SSL_CTX min version\n", i);
 				failed++;
 			}
 			goto next;
 		}
-		if (SSL_CTX_set_max_proto_version(ssl_ctx, mmvt->maxver) != 1) {
-			if (mmvt->want_maxver != 0) {
+		if (!SSL_CTX_set_max_proto_version(ssl_ctx, mmvt->maxver)) {
+			if (!mmvt->want_max_fail) {
 				fprintf(stderr, "FAIL: test %zu - failed to set "
 				    "SSL_CTX min version\n", i);
 				failed++;
@@ -683,13 +778,13 @@ test_ssl_min_max_version(void)
 			goto next;
 		}
 
-		if (mmvt->want_minver == 0) {
+		if (mmvt->want_min_fail) {
 			fprintf(stderr, "FAIL: test %zu - successfully set "
 			    "SSL_CTX min version, should have failed\n", i);
 			failed++;
 			goto next;
 		}
-		if (mmvt->want_maxver == 0) {
+		if (mmvt->want_max_fail) {
 			fprintf(stderr, "FAIL: test %zu - successfully set "
 			    "SSL_CTX max version, should have failed\n", i);
 			failed++;
@@ -731,16 +826,16 @@ test_ssl_min_max_version(void)
 			goto next;
 		}
 
-		if (SSL_set_min_proto_version(ssl, mmvt->minver) != 1) {
-			if (mmvt->want_minver != 0) {
+		if (!SSL_set_min_proto_version(ssl, mmvt->minver)) {
+			if (mmvt->want_min_fail) {
 				fprintf(stderr, "FAIL: test %zu - failed to set "
 				    "SSL min version\n", i);
 				failed++;
 			}
 			goto next;
 		}
-		if (SSL_set_max_proto_version(ssl, mmvt->maxver) != 1) {
-			if (mmvt->want_maxver != 0) {
+		if (!SSL_set_max_proto_version(ssl, mmvt->maxver)) {
+			if (mmvt->want_max_fail) {
 				fprintf(stderr, "FAIL: test %zu - failed to set "
 				    "SSL min version\n", i);
 				failed++;
@@ -748,13 +843,13 @@ test_ssl_min_max_version(void)
 			goto next;
 		}
 
-		if (mmvt->want_minver == 0) {
+		if (mmvt->want_min_fail) {
 			fprintf(stderr, "FAIL: test %zu - successfully set SSL "
 			    "min version, should have failed\n", i);
 			failed++;
 			goto next;
 		}
-		if (mmvt->want_maxver == 0) {
+		if (mmvt->want_max_fail) {
 			fprintf(stderr, "FAIL: test %zu - successfully set SSL "
 			    "max version, should have failed\n", i);
 			failed++;
@@ -803,5 +898,5 @@ main(int argc, char **argv)
 	if (failed == 0)
 		printf("PASS %s\n", __FILE__);
 
-        return (failed);
+	return (failed);
 }
